'use strict';

const metricsName = {
  FP: "first-paint",
  FCP: "first-contentful-paint",
  LCP: "largest-contentful-paint",
  FID: "first-input-delay",
  CLS: "cumulative-layout-shift",
  NT: "navigation-timing",
  RF: "resource-flow",
};

function observe(type, handler) {
  new PerformanceObserver((entryList) => {
    const entries = entryList.getEntries();
    const entry = entries[entries.length - 1];
    console.log(`${type}`, entry);
    handler(entry);
  }).observe({ type: type, buffered: true });
}

// export const observe = (type, callback) => {
//   // 类型合规，就返回 observe
//   if (PerformanceObserver.supportedEntryTypes?.includes(type)) {
//     const ob: PerformanceObserver = new PerformanceObserver((l) => l.getEntries().map(callback));

//     ob.observe({ type, buffered: true });
//     return ob;
//   }
//   return undefined;
// };

// function observe(type, handler) {
//   new PerformanceObserver((entryList) => {
//     const entries = entryList.getEntries();
//     const entry = entries[entries.length - 1];
//     console.log(`${type}`, entry);
//     handler(entry);
//   }).observe({ type: type, buffered: true });
// }

class WebVitals {
  
  constructor () {
    this.metrics = new MetricsStore();
    this.initLCP();
    this.initCLS();
    this.initResourceFlow();
    afterLoad(()=>{
      this.initNavigationTiming();
      this.initFP();
      this.initFCP();
      this.initFID();
      this.perfSendHandler();
    });
  };

  // 性能上报的策略
  perfSendHandler(){};

  // 初始化 FP 的获取以及返回
  initFP(){
    const getFP = () => {
      const [entry] = performance.getEntriesByName("first-paint");
      return entry;
    };
    const entry = getFP();
    const metrics = {
      startTime: entry?.startTime.toFixed(2),
      entry,
    };
    this.metrics.set(metricsName.FP, metrics);
  };

  // 初始化 FCP 的获取以及返回
  initFCP(){
    const getFCP = () => {
      const [entry] = performance.getEntriesByName("first-contentful-paint");
      return entry;
    };
    const entry = getFCP();
    const metrics = {
      startTime: entry?.startTime.toFixed(2),
      entry,
    };
    this.metrics.set(metricsName.FCP, metrics);
  };

  // 初始化 LCP 的获取以及返回
  initLCP(){
    const getLCP = (entryHandler) => {
      return observe("largest-contentful-paint", entryHandler);
    };
    const entryHandler = (entry) => {
      const metrics = {
        startTime: entry?.startTime.toFixed(2),
        entry,
      };
      this.metrics.set(metricsName.LCP, metrics);
    };
    getLCP(entryHandler);
  };

  // 初始化 FID 的获取 及返回
  initFID(){
    const getFID = (entryHandler) => {
      return observe("first-input", entryHandler);
    };
    const entryHandler = (entry) => {
      const metrics = {
        delay: entry.processingStart - entry.startTime,
        entry,
      };
      this.metrics.set(metricsName.FID, metrics);
    };
    getFID(entryHandler);
  };

  // 初始化 CLS 的获取以及返回
  initCLS(){
    const getCLS = (entryHandler) => {
      return observe("layout-shift", entryHandler);
    };

    let clsValue = 0;
    let clsEntries = [];

    let sessionValue = 0;
    let sessionEntries = [];

    const entryHandler = (entry) => {
      if (!entry.hadRecentInput) {
        const firstSessionEntry = sessionEntries[0];
        const lastSessionEntry = sessionEntries[sessionEntries.length - 1];

        // 如果条目与上一条目的相隔时间小于 1 秒且
        // 与会话中第一个条目的相隔时间小于 5 秒，那么将条目
        // 包含在当前会话中。否则，开始一个新会话。
        if (
          sessionValue &&
          entry.startTime - lastSessionEntry.startTime < 1000 &&
          entry.startTime - firstSessionEntry.startTime < 5000
        ) {
          sessionValue += entry.value;
          sessionEntries.push(entry);
        } else {
          sessionValue = entry.value;
          sessionEntries = [entry];
        }

        // 如果当前会话值大于当前 CLS 值，
        // 那么更新 CLS 及其相关条目。
        if (sessionValue > clsValue) {
          clsValue = sessionValue;
          clsEntries = sessionEntries;

          // 记录 CLS 到 Map 里
          const metrics = {
            entry,
            clsValue,
            clsEntries,
          };
          this.metrics.set(metricsName.CLS, metrics);
        }
      }
    };
    getCLS(entryHandler);
  };

  // 初始化 NT 的获取以及返回
  initNavigationTiming(){
    const getNavigationTiming = () => {
      const resolveNavigationTiming = (entry) => {
        const {
          domainLookupStart,
          domainLookupEnd,
          connectStart,
          connectEnd,
          secureConnectionStart,
          requestStart,
          responseStart,
          responseEnd,
          domInteractive,
          domContentLoadedEventEnd,
          loadEventStart,
          fetchStart,
        } = entry;

        return {
          // 关键时间点
          FP: responseEnd - fetchStart,
          TTI: domInteractive - fetchStart,
          DomReady: domContentLoadedEventEnd - fetchStart,
          Load: loadEventStart - fetchStart,
          FirstByte: responseStart - domainLookupStart,
          // 关键时间段
          DNS: domainLookupEnd - domainLookupStart,
          TCP: connectEnd - connectStart,
          SSL: secureConnectionStart ? connectEnd - secureConnectionStart : 0,
          TTFB: responseStart - requestStart,
          Trans: responseEnd - responseStart,
          DomParse: domInteractive - responseEnd,
          Res: loadEventStart - domContentLoadedEventEnd,
        };
      };

      const navigation =
        // W3C Level2  PerformanceNavigationTiming
        // 使用了High-Resolution Time，时间精度可以达毫秒的小数点好几位。
        performance.getEntriesByType("navigation").length > 0
          ? performance.getEntriesByType("navigation")[0]
          : performance.timing; // W3C Level1  (目前兼容性高，仍然可使用，未来可能被废弃)。
      return resolveNavigationTiming(navigation);
    };

    const navigationTiming = getNavigationTiming();
    const metrics = navigationTiming;
    this.metrics.set(metricsName.NT, metrics);
  };

  // 初始化 RF 的获取以及返回
  initResourceFlow() {
    const getRF = (callback) => {
      const resource = performance.getEntriesByType("resource");
      let cacheQuantity = 0;
      resource.map((item) => {
        if (item.duration == 0 && item.transferSize !== 0) cacheQuantity++;
        const data = {
          ...item,
          name: item.name, //资源地址
          startTime: item.startTime, //开始时间
          responseEnd: item.responseEnd, //结束时间
          time: item.duration, //消耗时间
          initiatorType: item.initiatorType, //资源类型
          transferSize: item.transferSize, //传输大小
          //请求响应耗时 ttfb = item.responseStart - item.startTime
          //内容下载耗时 tran = item.responseEnd - item.responseStart
          //但是受到跨域资源影响。除非资源设置允许获取timing
        };
        callback(data);
        return data
      });
      console.log("缓存命中率", (cacheQuantity / resource.length).toFixed(2));
    };

    const getResourceFlow = (resourceFlow) => {

      const entryHandler = (entry) => {
        const {
          name,
          transferSize,
          initiatorType,
          startTime,
          responseEnd,
          domainLookupEnd,
          domainLookupStart,
          connectStart,
          connectEnd,
          secureConnectionStart,
          responseStart,
          requestStart,
        } = entry;
        resourceFlow.push({
          // name 资源地址
          name,
          // transferSize 传输大小
          transferSize,
          // initiatorType 资源类型
          initiatorType,
          // startTime 开始时间
          startTime,
          // responseEnd 结束时间
          responseEnd,
          // 贴近 Chrome 的近似分析方案，受到跨域资源影响
          dnsLookup: domainLookupEnd - domainLookupStart,
          initialConnect: connectEnd - connectStart,
          ssl: connectEnd - secureConnectionStart,
          request: responseStart - requestStart,
          ttfb: responseStart - requestStart,
          contentDownload: responseStart - requestStart,
        });
      };

      return getRF(entryHandler);
    };

    const resourceFlow = [];
    const resObserve = getResourceFlow(resourceFlow);

    const stopListening = () => {
      if (resObserve) {
        resObserve.disconnect();
      }
      const metrics = resourceFlow;
      this.metrics.set(metricsName.RF, metrics);
    };
    // 当页面 pageshow 触发时，中止
    window.addEventListener("pageshow", stopListening, {
      once: true,
      capture: true,
    });
  }
}


const afterLoad = (callback) => {
  if (document.readyState === "complete") {
    setTimeout(callback);
  } else {
    window.addEventListener("pageshow", callback, {
      once: true,
      capture: true,
    });
  }
};

// import * as monitor from 'art-monitor'
// console.log(monitor.monitor)


const monitor = {
  init() {
    console.log("Monitoring init...");
    // WebVitals
    console.log("WebVitals", new WebVitals());
  },
};

// module.exports = monitor

// monitor.init()

// export function monitor() {
//     console.log('Monitoring data...');
// }

module.exports = monitor;
//# sourceMappingURL=index.cjs.js.map
